--- nginx-0.5.17/src/http/modules/ngx_http_autoindex_module.c	2006-11-14 02:53:37.000000000 +0600
+++ nginx-0.5.17-add_auto_index_sort/src/http/modules/ngx_http_autoindex_module.c	2007-04-10 20:46:26.000000000 +0700
@@ -36,6 +36,8 @@
     ngx_flag_t     enable;
     ngx_flag_t     localtime;
     ngx_flag_t     exact_size;
+    ngx_flag_t     reverse_sort;
+    ngx_str_t      sort_by;
 } ngx_http_autoindex_loc_conf_t;
 
 
@@ -44,8 +46,25 @@
 #define NGX_HTTP_AUTOINDEX_NAME_LEN     50
 
 
-static int ngx_libc_cdecl ngx_http_autoindex_cmp_entries(const void *one,
-    const void *two);
+static ngx_int_t ngx_libc_cdecl
+    ngx_http_autoindex_cmp_entries_by_name(const void *one,
+                                           const void *two);
+static ngx_int_t ngx_libc_cdecl
+    ngx_http_autoindex_cmp_entries_by_name_reverse(const void *one,
+                                                   const void *two);
+static ngx_int_t ngx_libc_cdecl
+    ngx_http_autoindex_cmp_entries_by_size(const void *one,
+                                           const void *two);
+static ngx_int_t ngx_libc_cdecl
+    ngx_http_autoindex_cmp_entries_by_size_reverse(const void *one,
+                                                   const void *two);
+static ngx_int_t ngx_libc_cdecl
+    ngx_http_autoindex_cmp_entries_by_date(const void *one,
+                                           const void *two);
+static ngx_int_t ngx_libc_cdecl
+    ngx_http_autoindex_cmp_entries_by_date_reverse(const void *one,
+                                                   const void *two);
+
 static ngx_int_t ngx_http_autoindex_error(ngx_http_request_t *r,
     ngx_dir_t *dir, ngx_str_t *name);
 static ngx_int_t ngx_http_autoindex_init(ngx_conf_t *cf);
@@ -77,6 +96,20 @@
       offsetof(ngx_http_autoindex_loc_conf_t, exact_size),
       NULL },
 
+    { ngx_string("autoindex_reverse_sort"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_conf_set_flag_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_autoindex_loc_conf_t, reverse_sort),
+      NULL },
+
+    { ngx_string("autoindex_sort_by"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
+      ngx_conf_set_str_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_autoindex_loc_conf_t, sort_by),
+      NULL },
+
       ngx_null_command
 };
 
@@ -149,6 +182,7 @@
     ngx_array_t                     entries;
     ngx_http_autoindex_entry_t     *entry;
     ngx_http_autoindex_loc_conf_t  *alcf;
+    ngx_int_t                     (*cmp_routine)(const void *, const void *);
 
     static char  *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                                "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
@@ -365,9 +399,36 @@
     }
 
     if (entries.nelts > 1) {
+
+        if (!ngx_strcmp(alcf->sort_by.data, "date")) {
+
+            if (alcf->reverse_sort) {
+                cmp_routine = ngx_http_autoindex_cmp_entries_by_date_reverse;
+            } else {
+                cmp_routine = ngx_http_autoindex_cmp_entries_by_date;
+            }
+
+        } else if (!ngx_strcmp(alcf->sort_by.data, "size")) {
+
+            if (alcf->reverse_sort) {
+                cmp_routine = ngx_http_autoindex_cmp_entries_by_size_reverse;
+            } else {
+                cmp_routine = ngx_http_autoindex_cmp_entries_by_size;
+            }
+
+        } else {
+
+            if (alcf->reverse_sort) {
+                cmp_routine = ngx_http_autoindex_cmp_entries_by_name_reverse;
+            } else {
+                cmp_routine = ngx_http_autoindex_cmp_entries_by_name;
+            }
+
+        }
+
         ngx_qsort(entry, (size_t) entries.nelts,
                   sizeof(ngx_http_autoindex_entry_t),
-                  ngx_http_autoindex_cmp_entries);
+                  cmp_routine);
     }
 
     b->last = ngx_cpymem(b->last, title, sizeof(title) - 1);
@@ -522,8 +583,111 @@
 }
 
 
-static int ngx_libc_cdecl
-ngx_http_autoindex_cmp_entries(const void *one, const void *two)
+static ngx_int_t ngx_libc_cdecl
+ngx_http_autoindex_cmp_entries_by_name(const void *one, const void *two)
+{
+    ngx_http_autoindex_entry_t *first = (ngx_http_autoindex_entry_t *) one;
+    ngx_http_autoindex_entry_t *second = (ngx_http_autoindex_entry_t *) two;
+
+    if (first->dir && !second->dir) {
+        /* move the directories to the start */
+        return -1;
+    }
+
+    if (!first->dir && second->dir) {
+        /* move the directories to the start */
+        return 1;
+    }
+
+    return (ngx_int_t) ngx_strcmp(first->name.data, second->name.data);
+}
+
+
+static ngx_int_t ngx_libc_cdecl
+ngx_http_autoindex_cmp_entries_by_name_reverse(const void *one,
+                                               const void *two)
+{
+    ngx_http_autoindex_entry_t *first = (ngx_http_autoindex_entry_t *) one;
+    ngx_http_autoindex_entry_t *second = (ngx_http_autoindex_entry_t *) two;
+
+    if (first->dir && !second->dir) {
+        /* move the directories to the end */
+        return 1;
+    }
+
+    if (!first->dir && second->dir) {
+        /* move the directories to the end */
+        return -1;
+    }
+
+    return (ngx_int_t) ngx_strcmp(second->name.data, first->name.data);
+}
+
+
+static ngx_int_t ngx_libc_cdecl
+ngx_http_autoindex_cmp_entries_by_date(const void *one, const void *two)
+{
+    ngx_http_autoindex_entry_t *first = (ngx_http_autoindex_entry_t *) one;
+    ngx_http_autoindex_entry_t *second = (ngx_http_autoindex_entry_t *) two;
+
+    if (first->dir && !second->dir) {
+        /* move the directories to the start */
+        return -1;
+    }
+
+    if (!first->dir && second->dir) {
+        /* move the directories to the start */
+        return 1;
+    }
+
+    return (ngx_int_t) (first->mtime - second->mtime);
+}
+
+
+static ngx_int_t ngx_libc_cdecl
+ngx_http_autoindex_cmp_entries_by_date_reverse(const void *one,
+                                               const void *two)
+{
+    ngx_http_autoindex_entry_t *first = (ngx_http_autoindex_entry_t *) one;
+    ngx_http_autoindex_entry_t *second = (ngx_http_autoindex_entry_t *) two;
+
+    if (first->dir && !second->dir) {
+        /* move the directories to the start */
+        return -1;
+    }
+
+    if (!first->dir && second->dir) {
+        /* move the directories to the start */
+        return 1;
+    }
+
+    return (ngx_int_t) (second->mtime - first->mtime);
+}
+
+
+static ngx_int_t ngx_libc_cdecl
+ngx_http_autoindex_cmp_entries_by_size(const void *one, const void *two)
+{
+    ngx_http_autoindex_entry_t *first = (ngx_http_autoindex_entry_t *) one;
+    ngx_http_autoindex_entry_t *second = (ngx_http_autoindex_entry_t *) two;
+
+    if (first->dir && !second->dir) {
+        /* move the directories to the start */
+        return -1;
+    }
+
+    if (!first->dir && second->dir) {
+        /* move the directories to the start */
+        return 1;
+    }
+
+    return (ngx_int_t) (first->size - second->size);
+}
+
+
+static ngx_int_t ngx_libc_cdecl
+ngx_http_autoindex_cmp_entries_by_size_reverse(const void *one,
+                                               const void *two)
 {
     ngx_http_autoindex_entry_t *first = (ngx_http_autoindex_entry_t *) one;
     ngx_http_autoindex_entry_t *second = (ngx_http_autoindex_entry_t *) two;
@@ -538,7 +702,7 @@
         return 1;
     }
 
-    return (int) ngx_strcmp(first->name.data, second->name.data);
+    return (ngx_int_t) (second->size - first->size);
 }
 
 
@@ -605,6 +769,7 @@
     conf->enable = NGX_CONF_UNSET;
     conf->localtime = NGX_CONF_UNSET;
     conf->exact_size = NGX_CONF_UNSET;
+    conf->reverse_sort = NGX_CONF_UNSET;
 
     return conf;
 }
@@ -619,6 +784,8 @@
     ngx_conf_merge_value(conf->enable, prev->enable, 0);
     ngx_conf_merge_value(conf->localtime, prev->localtime, 0);
     ngx_conf_merge_value(conf->exact_size, prev->exact_size, 1);
+    ngx_conf_merge_value(conf->reverse_sort, prev->reverse_sort, 0);
+    ngx_conf_merge_str_value(conf->sort_by, prev->sort_by, "name");
 
     return NGX_CONF_OK;
 }